# _*_ coding: utf-8 _*_
import re
from django import forms
from bookings.models import User,TypCeny,Apartment
from django.core.exceptions import ObjectDoesNotExist

class FormularzRejestracji(forms.Form):
    username = forms.CharField(label=" Login:",max_length=30)
    email = forms.EmailField(label="Email:")
    password1 = forms.CharField(label="Hasło:",widget=forms.PasswordInput())
    password2 = forms.CharField(label="Powtórz hasło:",widget=forms.PasswordInput())
    phone = forms.CharField(label="Telefon:",max_length=20,required=False)
    log_on = forms.BooleanField(label="Logowanie po rejestracji:",required=False)    
    def clean_password2(self):
        password1 = self.cleaned_data['password1']
        password2 = self.cleaned_data['password2']
        if password1 == password2:
            return password2
        else:
            raise forms.ValidationError("Hasła się różnią")                  
    def clean_username(self):
        username = self.cleaned_data['username']
        if not re.search(r'^\w+$',username):
            raise forms.ValidationError("Dopuszczalne są tylko cyfry, litery angielskie i _")
        try:
            User.objects.get(username=username)
        except ObjectDoesNotExist:
            return username
        raise forms.ValidationError("Taki użytkownik już istnieje")

class FormularzLogowania(forms.Form):
    username = forms.CharField(label="Login:",max_length=30)
    password = forms.CharField(label="Hasło:",widget=forms.PasswordInput())
    def clean_username(self):
        username = self.cleaned_data['username']
        try:
            user = User.objects.get(username=username)
            return username
        except ObjectDoesNotExist:
            raise forms.ValidationError("Niepoprawny login")        
    def clean_password(self):
        if self.cleaned_data.has_key('username'):
            username = self.cleaned_data['username']
            password = self.cleaned_data['password']
            user = User.objects.get(username=username)
            if user.check_password(password):
                return password
            raise forms.ValidationError("Niepoprawne hasło")                
        raise forms.ValidationError("Niepoprawny login")
             
class FormularzEdycjiTypu(forms.Form):
    def __init__(self,dictionary,control):
        forms.Form.__init__(self,dictionary)
        self.control = control
    id = forms.IntegerField(required=False,widget=forms.HiddenInput())        
    description = forms.CharField(label="Opis:",max_length=60,widget=forms.TextInput({'size':60}))
    active = forms.BooleanField(label="Aktywna:",required=False)
    def clean_description(self):
        description = self.cleaned_data['description']    
        if not self.control:
            return description
        id = self.cleaned_data['id']
        if len(description) == 0:
            raise forms.ValidationError("To pole musi być wypełnione")           
        try:
            type = TypCeny.objects.get(description=description)
            if type.id == id:
                return description
        except ObjectDoesNotExist:             
            return description
        raise forms.ValidationError("Taki opis już występuje")            
    
class FormularzEdycjiApartamentu(forms.Form):
    name = forms.CharField(label="Opis:",max_length=80,widget=forms.TextInput({'size':80}))
    bedrooms = forms.IntegerField(label="Sypialnie:",min_value=1,max_value=4)
    beds = forms.IntegerField(label="Miejsca:",min_value=1,max_value=12)
    kitchenette = forms.BooleanField(label="Aneks kuchenny:",required=False)
    file = forms.FileField(label="Zdjęcie:",required=False)

class FormularzPocztowy(forms.Form):
    subject = forms.CharField(label="Temat (min. 10 znaków):",min_length=10,max_length=100,widget=forms.TextInput({'size':100}))
    body = forms.CharField(label="Treść:",widget=forms.Textarea({'rows':10,'cols':60}))    
    
class FormularzEdycjiOsoby(forms.Form):
    def __init__(self,dictionary,control):
        forms.Form.__init__(self,dictionary)
        self.control = control    
    id = forms.IntegerField(required=False,widget=forms.HiddenInput())    
    username = forms.CharField(label=" Login:",max_length=30)
    email = forms.EmailField(label="Email:")
    phone = forms.CharField(label="Telefon:",max_length=20,required=False) 
    def clean_username(self):
        username = self.cleaned_data['username']
        if not self.control:
            return username
        id = self.cleaned_data['id']            
        if not re.search(r'^\w+$',username):
            raise forms.ValidationError("Dopuszczalne są tylko cyfry, litery angielskie oraz _")
        print id,username    
        try:
            User.objects.exclude(id=id).get(username=username)
        except ObjectDoesNotExist:
            return username
        raise forms.ValidationError("Taki użytkownik już istnieje")
        
class FormularzRezerwacji(forms.Form):
    from_day = forms.DateField(label="Od dnia (format dd.mm.rr):",input_formats=['%d.%m.%y'],widget=forms.DateInput(format='%d.%m.%y'))
    to_day = forms.DateField(label="Do dnia (format dd.mm.rr):",input_formats=['%d.%m.%y'],widget=forms.DateInput(format='%d.%m.%y'))
    def clean_to_day(self):
        from_day = self.cleaned_data['from_day']
        to_day = self.cleaned_data['to_day']
        if from_day<=to_day:
            return to_day
        raise forms.ValidationError("Koniec jest wcześniejszy niż początek")

class FormularzKorektyRezerwacji(forms.Form):
    from_day = forms.DateField(label="Od dnia (format dd.mm.rr):",input_formats=['%d.%m.%y'],widget=forms.DateInput(format='%d.%m.%y'))
    to_day = forms.DateField(label="Do dnia (format dd.mm.rr):",input_formats=['%d.%m.%y'],widget=forms.DateInput(format='%d.%m.%y'))
    apartment = forms.IntegerField(label="Numer apartamentu:")
    def clean_to_day(self):
        from_day = self.cleaned_data['from_day']
        to_day = self.cleaned_data['to_day']
        if from_day<=to_day:
            return to_day
        raise forms.ValidationError("Koniec jest wcześniejszy niż początek")
    def clean_apartment(self):
        apartment = self.cleaned_data['apartment']
        try:
            Apartment.objects.get(id=int(apartment))
            return apartment
        except ObjectDoesNotExist:
            raise forms.ValidationError("Nie ma takiego apartamentu")         
           
