﻿# _*_ coding: utf-8 _*_
from django.http import HttpResponse,HttpResponseRedirect
from django.template.loader import get_template
from django.template import RequestContext
from django.shortcuts import *
from django.contrib.auth import login,authenticate,logout
from django.contrib.auth.decorators import permission_required,login_required
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import send_mail,BadHeaderError
from datetime import *
from bookings.models import Apartment,User,Cena,TypCeny,Booking
from bookings.forms import *

def main_page(request):
    template = get_template("main_page.html")
    apartments = Apartment.objects.all()
    variables = RequestContext(request,{'podpis':'Widok od strony rzeki','apartments':apartments})
    output = template.render(variables)
    return HttpResponse(output) 

def register_page(request):
    if request.method == 'POST':
        form = FormularzRejestracji(request.POST)
        if form.is_valid():
            user = User.objects.create_user(
              username=form.cleaned_data['username'],
              password=form.cleaned_data['password1'],
              email=form.cleaned_data['email']
            )
            user.last_name = form.cleaned_data['phone']
            user.save()
            if form.cleaned_data['log_on']:
                user = authenticate(username=form.cleaned_data['username'],password=form.cleaned_data['password1'])
                login(request,user)
                template = get_template("main_page.html")
                variables = RequestContext(request,{'user':user})
                output = template.render(variables)
                return HttpResponseRedirect("/") 
            else:    
                template = get_template("registration/register_success.html")
                variables = RequestContext(request,{'username':form.cleaned_data['username']})
                output = template.render(variables)
                return HttpResponse(output) 
    else:
        form = FormularzRejestracji()
    template = get_template("registration/register.html")    
    variables = RequestContext(request,{'form':form})
    output = template.render(variables)
    return HttpResponse(output)
    
@permission_required('user.is_superuser',"/login/")     
def admin_page(request):
    template = get_template("admin.html")
    apartments = Apartment.objects.all()
    types = TypCeny.objects.all()
    users = User.objects.order_by('username')
    if request.method == 'POST' and request.POST.has_key('first'):
        first = int(request.POST['first'])
        if request.POST.has_key('back'):
            first = first - 10
        if request.POST.has_key('forward'):
            first = first + 10            
    else:
        first = 0
    variables = RequestContext(request,{'apartments':apartments,'types':types,'users':users[first:first+10],'number_of_users':len(users),'first':first,'end':(first+10>=len(users))})                
    output = template.render(variables)            
    return HttpResponse(output)     

def login_page(request):
    if request.method == 'POST':
        form = FormularzLogowania(request.POST)
        if form.is_valid():
            user = authenticate(username=form.cleaned_data['username'],password=form.cleaned_data['password'])
            login(request,user)
            if user.is_superuser:
                template = get_template("admin.html")
                apartments = Apartment.objects.all()
                types = TypCeny.objects.all()
                users = User.objects.order_by('username')
                if request.method == 'POST' and request.POST.has_key('first'):
                    first = int(request.POST['first'])
                    if request.POST.has_key('back'):
                        first = first - 10
                    if request.POST.has_key('forward'):
                        first = first + 10            
                else:
                    first = 0               
                variables = RequestContext(request,{'apartments':apartments,'types':types,'users':users[first:first+10],'number_of_users':len(users),'first':first,'end':(first+10>=len(users))})                
                output = template.render(variables)            
                return HttpResponse(output) 
            else:
                template = get_template("main_page.html")
                variables = RequestContext(request,{'user':user})
                output = template.render(variables)
                return HttpResponseRedirect("/")                         
    else: 
        form = FormularzLogowania()
    template = get_template("registration/login.html")    
    variables = RequestContext(request,{'form':form})
    output = template.render(variables)
    return HttpResponse(output)

def logout_page(request):
    logout(request)
    return HttpResponseRedirect("/")
    
def prices_page(request,id_app):     
    template = get_template("prices.html")
    apartment = Apartment.objects.get(id=id_app)
    prices = Cena.objects.select_related().filter(apartment=id_app)
    variables = RequestContext(request,{'apartment':apartment,'prices':prices})
    output = template.render(variables)
    return HttpResponse(output)

@permission_required('user.is_superuser',"/login/")     
def all_prices_page(request,id_app):
    legend = ''
    if request.method == 'POST':
        for key in request.POST.keys():
            ok = True    
            if key.isdigit():
                id_type = int(key)
                try:
                    price_value = int(request.POST[key])
                    save = (price_value>0)
                    ok = (price_value>=0)
                except ValueError:
                    ok = False
            else:
                ok = False
            if ok:
                legend = 'Nowe ceny zostały zapisane'
                if save:
                    try:
                        cena = Cena.objects.get(type=id_type,apartment=id_app)
                        cena.price = price_value
                    except ObjectDoesNotExist:    
                        cena = Cena(price=price_value,type_id=id_type,apartment_id=id_app)
                        cena = Cena.objects.create(price=price_value,type=TypCeny.objects.get(id=id_type),apartment=Apartment.objects.get(id=id_app))
                    cena.save()
                else:
                    try:
                        cena = Cena.objects.get(type=id_type,apartment=id_app)
                        cena.delete()
                    except ObjectDoesNotExist:
                        pass    
    template = get_template("edition/prices_edition.html")
    apartment = Apartment.objects.get(id=id_app)
    types = TypCeny.objects.all()
    for typ in types:
        try:
            typ.price = Cena.objects.select_related().get(apartment=id_app,type=typ.id).price
        except ObjectDoesNotExist:
            typ.price = 0        
    variables = RequestContext(request,{'apartment':apartment,'types':types,'legend':legend})
    output = template.render(variables)
    return HttpResponse(output)    

@permission_required('user.is_superuser',"/login/")    
def save_type_page(request):
    if request.method == 'POST':
        form = FormularzEdycjiTypu(request.POST,True)
        if form.is_valid():
            id = request.POST['id']
            if len(id) == 0:
                type = TypCeny() 
            else:
                type = TypCeny.objects.get(id=int(id))
            type.description = form.cleaned_data['description']
            type.active = form.cleaned_data['active']
            type.save()
            apartments = Apartment.objects.all()
            types = TypCeny.objects.all()
            first = 0
            users = User.objects.order_by('username')
            template = get_template("admin.html")        
            variables = RequestContext(request,{'apartments':apartments,'types':types,'users':users[first:first+10],'number_of_users':len(users),'first':first,'end':(first+10>=len(users))})                            
            output = template.render(variables)            
            return HttpResponse(output)                     
    elif request.GET.has_key('id'):
        id = request.GET['id']
        type = TypCeny.objects.get(id=id)
        form = FormularzEdycjiTypu({'description':type.description,'active':type.active},False)
        template = get_template("edition/type_edition.html")
        variables = RequestContext(request,{'form':form,'type':type})
        output = template.render(variables)
        return HttpResponse(output)        
    else:
        form = FormularzEdycjiTypu({'active':True},False)
    template = get_template("edition/type_edition.html")
    variables = RequestContext(request,{'form':form})
    output = template.render(variables)
    return HttpResponse(output)                

@permission_required('user.is_superuser',"/login/")    
def save_apartment_page(request):
    if request.method == 'POST':
        form = FormularzEdycjiApartamentu(request.POST,request.FILES)
        if form.is_valid():
            id = request.POST['id']
            if len(id) == 0:
                apartment = Apartment() 
            else:
                apartment = Apartment.objects.get(id=int(id))
            apartment.name = form.cleaned_data['name']
            apartment.kitchenette = form.cleaned_data['kitchenette']
            apartment.bedrooms = form.cleaned_data['bedrooms']
            apartment.beds = form.cleaned_data['beds']
            if request.FILES:
                f = request.FILES['file']
                destination = open("media/"+str(f),'wb+')
                for chunk in f.chunks():
                    destination.write(chunk)
                destination.close()
                apartment.pathToPicture = str(f)
            apartment.save()
            apartments = Apartment.objects.all()
            types = TypCeny.objects.all()
            first = 0
            users = User.objects.order_by('username')
            template = get_template("admin.html")        
            variables = RequestContext(request,{'apartments':apartments,'types':types,'users':users[first:first+10],'number_of_users':len(users),'first':first,'end':(first+10>=len(users))})                            
            output = template.render(variables)            
            return HttpResponse(output)                     
    elif request.GET.has_key('id'):
        ap_type = int(request.GET['id'])
        apartment = Apartment.objects.get(id=ap_type)
        form = FormularzEdycjiApartamentu({'name':apartment.name,'bedrooms':apartment.bedrooms,'beds':apartment.beds,'kitchenette':apartment.kitchenette})
        template = get_template("edition/apartment_edition.html")
        variables = RequestContext(request,{'form':form,'apartment':apartment})
        output = template.render(variables)
        return HttpResponse(output)        
    else:
        form = FormularzEdycjiApartamentu({'kitchenette':True})
    template = get_template("edition/apartment_edition.html")
    variables = RequestContext(request,{'form':form})
    output = template.render(variables)
    return HttpResponse(output) 

@permission_required('user.is_superuser',"/login/")    
def delete_type_page(request):
    if request.method == 'POST':
        id = request.POST['id']
        type = TypCeny.objects.get(id=id)
        type.delete()
        apartments = Apartment.objects.all()
        types = TypCeny.objects.all()
        first = 0
        users = User.objects.order_by('username')
        template = get_template("admin.html")        
        variables = RequestContext(request,{'apartments':apartments,'types':types,'users':users[first:first+10],'number_of_users':len(users),'first':first,'end':(first+10>=len(users))})                        
        output = template.render(variables)            
        return HttpResponse(output)                     
    else:
        id = request.GET['id']
        type = TypCeny.objects.get(id=id)
        template = get_template("edition/type_delete.html")
        variables = RequestContext(request,{'type':type})
        output = template.render(variables)
        return HttpResponse(output)        

@permission_required('user.is_superuser',"/login/")     
def send_mail_page(request):
    template = get_template("send_mail.html")
    if request.method == 'POST':
        id = request.POST['sender']
        sender = User.objects.get(id=id)
        id = request.POST['recipient']
        recipient = User.objects.get(id=id)
        form = FormularzPocztowy(request.POST)
        if form.is_valid():
            try:
                send_mail(form.cleaned_data['subject'],form.cleaned_data['body'],sender.email,[recipient.email])
                return HttpResponseRedirect("/admin/")
            except BadHeaderError:    
                pass
    else:
        id = request.GET['id']     
        user = User.objects.get(id=id)
        form = FormularzPocztowy()
    variables = RequestContext(request,{'form':form,'recipient':user})                
    output = template.render(variables)            
    return HttpResponse(output)

@permission_required('user.is_superuser',"/login/")         
def save_user_page(request,args):
    if request.method == 'POST':
        form = FormularzEdycjiOsoby(request.POST,True)
        if form.is_valid():
            id = request.POST['id']
            user = User.objects.get(id=id)
            user.username = form.cleaned_data['username']
            user.email = form.cleaned_data['email']
            user.last_name = form.cleaned_data['phone']
            user.save()
            return HttpResponseRedirect("/admin/")
    else:
        id = int(args)
        user = User.objects.get(id=id)
        form = FormularzEdycjiOsoby({'id':id,'username':user.username,'phone':user.last_name,'email':user.email},False)    
    template = get_template("edition/user_edition.html")
    variables = RequestContext(request,{'form':form})                
    output = template.render(variables)            
    return HttpResponse(output)
    
@permission_required('user.is_superuser',"/login/")    
def delete_user_page(request,args):
    if request.method == 'POST':
        id = request.POST['id']
        user = User.objects.get(id=id)
        user.delete()
        apartments = Apartment.objects.all()
        types = TypCeny.objects.all()
        first = 0
        users = User.objects.order_by('username')
        template = get_template("admin.html")        
        variables = RequestContext(request,{'apartments':apartments,'types':types,'users':users[first:first+10],'number_of_users':len(users),'first':first,'end':(first+10>=len(users))})                
        output = template.render(variables)            
        return HttpResponse(output)                     
    else:
        id = args
        user = User.objects.get(id=id)
        day = date.today()
        number_of_bookings = Booking.objects.filter(who=user,date__gte=day).count()
        template = get_template("edition/user_delete.html")
        variables = RequestContext(request,{'deleting_user':user,'bookings':number_of_bookings})
        output = template.render(variables)
        return HttpResponse(output)    
         
def one_room_page(request,args):
    year = date.today().year
    month = date.today().month    
    if request.GET.has_key('year'):
        year = int(request.GET['year'])
    if request.GET.has_key('month'):
        month = int(request.GET['month'])
    admin = (args == '1')        
    id = int(request.GET['id'])
    apartment = Apartment.objects.get(id=id)
    next = next_month_year(month,year)
    prev = prev_month_year(month,year)
    weeks = []
    week =[]
    day = last_monday(month,year)
    last_day = first_sunday(month,year)
    while day<=last_day:
        try:
            booking = Booking.objects.get(apartment=apartment,date=day)
            week.append(Day(day,color="#7AE68E",content=booking.who.username))
        except ObjectDoesNotExist:
            week.append(Day(day))        
        if len(week) == 7:
            weeks.append(week)
            week = []
        day+=timedelta(days=1)
    template = get_template("show_one_room.html")
    variables = RequestContext(request,{'apartment':apartment,'month':month,'year':year,'next_month':str(next[0]),'next_year':str(next[1]),'prev_month':str(prev[0]),'prev_year':str(prev[1]),'weeks':weeks,'admin':admin})
    output = template.render(variables)
    return HttpResponse(output)
    
def all_rooms_page(request,args):
    year = date.today().year
    month = date.today().month    
    if request.GET.has_key('year'):
        year = int(request.GET['year'])
    if request.GET.has_key('month'):
        month = int(request.GET['month'])
    admin = (args == '1')        
    next = next_month_year(month,year)
    prev = prev_month_year(month,year)
    weeks = []
    week =[]
    day = last_monday(month,year)
    last_day = first_sunday(month,year)
    number_of_rooms = Apartment.objects.count()
    while day<=last_day:
        bookings = Booking.objects.filter(date=day)
        if len(bookings) == 0:
            week.append(Day(day,content=str(number_of_rooms)))
        elif len(bookings) == number_of_rooms:
            week.append(Day(day,color="#ff0000"))
        else:        
            week.append(Day(day,color="#7ae68e",content=str(number_of_rooms-len(bookings))))
        if len(week) == 7:
            weeks.append(week)
            week = []
        day+=timedelta(days=1)
    template = get_template("show_all_rooms.html")
    variables = RequestContext(request,{'month':month,'year':year,'next_month':str(next[0]),'next_year':str(next[1]),'prev_month':str(prev[0]),'prev_year':str(prev[1]),'weeks':weeks,'admin':admin})
    output = template.render(variables)
    return HttpResponse(output)

@login_required
def booking_page(request,args):
    id = int(args)
    id_user = int(request.GET['id'])    
    if request.method == 'POST':
        form = FormularzRezerwacji(request.POST)
        if form.is_valid():
            first_day = datetime.strptime(request.POST['from_day'],'%d.%m.%y')
            last_day = datetime.strptime(request.POST['to_day'],'%d.%m.%y')
            if free_room(id,first_day,last_day):
                if save_new_booking(id,first_day,last_day,id_user):
                    template = get_template("edition/booking_success.html")
                    variables = RequestContext(request)
                    output = template.render(variables)
                    return HttpResponse(output)
            else:
                apartment = Apartment.objects.get(id=id)
                template = get_template("edition/booking_failure.html")
                variables = RequestContext(request,{'apartment':apartment,'id':id_user,'from':request.POST['from_day'],'to':request.POST['to_day']})            
                output = template.render(variables)
                return HttpResponse(output)                            
    else:
        form = FormularzRezerwacji()    
    apartment = Apartment.objects.get(id=id)
    template = get_template("edition/booking.html")
    variables = RequestContext(request,{'apartment':apartment,'form':form,'id':id_user})
    output = template.render(variables)
    return HttpResponse(output)

@login_required
def booking_correction_page(request,arg):
    args = arg.split('_')
    id_user = int(args[0])
    weeks = []
    week = []
    if request.method == 'POST':
        form = FormularzKorektyRezerwacji(request.POST)
        id = int (request.POST['apartment'])
        first_day = datetime.strptime(request.POST['from_day'],'%d.%m.%y')
        last_day = datetime.strptime(request.POST['to_day'],'%d.%m.%y') 
        if form.is_valid():
            if free_room(id,first_day,last_day):
                if save_new_booking(id,first_day,last_day,id_user):
                    template = get_template("edition/booking_success.html")
                    variables = RequestContext(request)
                    output = template.render(variables)
                    return HttpResponse(output)
            else:
                template = get_template("main_page.html")
                variables = RequestContext(request)            
                output = template.render(variables)
                return HttpResponse(output)
        else:
            first_day = last_monday_before(first_day-timedelta(days=3))
            last_day = first_sunday_after(last_day+timedelta(days=3))
    else:
        id = int(args[0])
        first_day = last_monday_before(datetime.strptime(args[1],'%d.%m.%y')-timedelta(days=3))
        last_day = first_sunday_after(datetime.strptime(args[2],'%d.%m.%y')+timedelta(days=3))        
        form = FormularzKorektyRezerwacji({'from_day':args[1],'to_day':args[2]})
    number_of_rooms = Apartment.objects.count()
    day = first_day
    print day,last_day,(day<=last_day)
    while day<=last_day:
        bookings = Booking.objects.filter(date=day)
        if len(bookings) == 0:
            wolne = ""
            for apartment in Apartment.objects.all():
                if len(Booking.objects.filter(date=day,apartment=apartment))==0:
                    wolne+=str(apartment.id)+" "        
            week.append(Day(day,content=wolne))
        elif len(bookings) == number_of_rooms:
            week.append(Day(day,color="#ff0000"))
        else:
            wolne = ""
            for apartment in Apartment.objects.all():
                if len(Booking.objects.filter(date=day,apartment=apartment))==0:
                    wolne+=str(apartment.id)+" "
            week.append(Day(day,color="#7ae68e",content=wolne))
        if len(week) == 7:
            weeks.append(week)
            week = []
        day+=timedelta(days=1)    
    template = get_template("edition/booking_correction.html")
    variables = RequestContext(request,{'id':id_user,'weeks':weeks,'from':datetime.strftime(first_day,'%d.%m.%y'),'to':datetime.strftime(last_day,'%d.%m.%y'),'form':form})
    output = template.render(variables)
    return HttpResponse(output)

    
def next_month_year(month,year):
    if month == 12:
        return (1,year+1)
    else:
        return (month+1,year)
        
def prev_month_year(month,year):
    if month == 1:
        return (12,year-1)
    else:
        return (month-1,year)
        
def last_monday(month,year):
    d = date(year,month,1)
    while d.weekday()>0:
        d-=timedelta(days=1)
    return d
    
def last_monday_before(d):
    while d.weekday()>0:
        d-=timedelta(days=1)
    return d    
    
def first_sunday(month,year):
    next = next_month_year(month,year)
    d = date(next[1],next[0],1)
    while d.weekday()<6:
        d+=timedelta(days=1)
    return d

def first_sunday_after(d):
    while d.weekday()<6:
        d+=timedelta(days=1)
    return d    

class Day:
    def __init__(self,date,color="#fdf5e5",content=""):
        self.date = date
        self.content = content
        self.color = color
        self.header = date.strftime('%d.%m.%y')
        
def free_room(apartment,first,last):
    day = first
    while day<=last:
        try:
            Booking.objects.get(apartment=apartment,date=day)
            return False
        except ObjectDoesNotExist:
            day+=timedelta(days=1)
    return True        
         
def save_new_booking(apartment,first,last,user):
    day = first
    while day<=last:
        ap = Apartment.objects.get(id=apartment)
        us = User.objects.get(id=user)
        booking = Booking.objects.create(apartment=ap,date=day,who=us)
        booking.save()
        day+=timedelta(days=1)
    return True        